@@ -48,7 +48,7 @@ BOOL cmd_write(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
 
         if(sCommand_option_item(command, "-error")) {
-            if(!bufsiz_write(fd, SFD(gStderr).fdbuf.mBuf)) {
+            if(!bufsiz_write(fd, SFD(gStderr).fdbuf.mBuf, SFD(gStderr).fdbuf.mBufLen)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -56,7 +56,7 @@ BOOL cmd_write(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             fd_clear(gStderr);
         }
         else {
-            if(!bufsiz_write(fd, SFD(nextin).fdbuf.mBuf)) {
+            if(!bufsiz_write(fd, SFD(nextin).fdbuf.mBuf, SFD(nextin).fdbuf.mBufLen)) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
@@ -65,7 +65,7 @@ BOOL cmd_write(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         (void)close(fd);
 
-        if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+        if(SFD(nextin).fdbuf.mBufLen == 0) {
             runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
         }
         else {
@@ -279,12 +279,12 @@ BOOL cmd_pushd(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(S_ISDIR(stat_.st_mode)) {
                     vector_add(gDirStack, STRING_NEW_GC(path2, FALSE));
 
-                    if(!fd_write(nextout, path2)) {
+                    if(!fd_write(nextout, path2, strlen(path2))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
-                    if(!fd_write(nextout, "\n")) {
+                    if(!fd_write(nextout, "\n", 1)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
