@@ -38,9 +38,9 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char*obj_kind[16] = {
             NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
         };
-        snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
+        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
 
-        if(!fd_write(nextout, buf)) {
+        if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -166,7 +166,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -179,13 +179,13 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* fun = access_object(command->mArgsRuntime[1], &reciever, runinfo->mRunningObject);
             if(fun && TYPE(fun) == T_FUN) {
                 char* source = SBLOCK(fun).mSource;
-                if(!fd_write(nextout, source)) {
+                if(!fd_write(nextout, source, strlen(source))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(source[strlen(source)-1] != '\n') {   /// pomch
-                    if(!fd_write(nextout, "\n")) {
+                    if(!fd_write(nextout, "\n", 1)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -234,7 +234,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -247,13 +247,13 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* klass = access_object(command->mArgsRuntime[1], &reciever, runinfo->mRunningObject);
             if(klass && TYPE(klass) == T_CLASS) {
                 char* source = SBLOCK(klass).mSource;
-                if(!fd_write(nextout, source)) {
+                if(!fd_write(nextout, source, strlen(source))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(source[strlen(source)-1] != '\n') {   /// pomch
-                    if(!fd_write(nextout, "\n")) {
+                    if(!fd_write(nextout, "\n", 1)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -405,12 +405,12 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                             char* pos = str_kanjipos2pointer(code, string_c_str(var), num);
                             char* pos2 = str_kanjipos2pointer(code, pos, 1);
-                            if(!fd_write3(nextout, pos, pos2-pos)) {
+                            if(!fd_write(nextout, pos, pos2-pos)) {
                                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -432,12 +432,12 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* reciever = runinfo->mCurrentObject;
                     sObject* var = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
                     if(var && TYPE(var) == T_STRING) {
-                        if(!fd_write(nextout, string_c_str(var))) {
+                        if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
                         }
-                        if(!fd_write(nextout, field)) {
+                        if(!fd_write(nextout, field, strlen(field))) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
@@ -479,7 +479,8 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
-                    if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i-1))) {
+                    char* str = vector_item(SFD(nextin).fdbuf.mLines, i-1);
+                    if(!fd_write(nextout, str, strlen(str))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -487,7 +488,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -578,12 +579,12 @@ BOOL cmd_pwo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     sObject* object = gCurrentObject;
     while(object != gRootObject) {
-        if(!fd_write(nextout, SUOBJECT(object).mName)) {
+        if(!fd_write(nextout, SUOBJECT(object).mName, strlen(SUOBJECT(object).mName))) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
-        if(!fd_write(nextout, "<-")) {
+        if(!fd_write(nextout, "<-", 2)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -592,13 +593,13 @@ BOOL cmd_pwo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         object = SUOBJECT(object).mParent;
     }
 
-    if(!fd_write(nextout, SUOBJECT(object).mName)) {
+    if(!fd_write(nextout, SUOBJECT(object).mName, strlen(SUOBJECT(object).mName))) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
     }
 
-    if(!fd_write(nextout, "\n")) {
+    if(!fd_write(nextout, "\n", 1)) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
         return FALSE;
@@ -659,13 +660,13 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* item = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
                 if(item) {
                     char buf[128];
-                    snprintf(buf, 128, "%p", item);
-                    if(!fd_write(nextout, buf)) {
+                    int size = snprintf(buf, 128, "%p", item);
+                    if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
-                    if(!fd_write(nextout, field)) {
+                    if(!fd_write(nextout, field, strlen(field))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -701,9 +702,9 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     char* obj_kind[16] = {
                         NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "job", "object", "memchecker", "function", "class", "external program", "completion", "external object"
                     };
-                    snprintf(buf, 128, "%s\n", obj_kind[type]);
+                    int size = snprintf(buf, 128, "%s\n", obj_kind[type]);
 
-                    if(!fd_write(nextout, buf)) {
+                    if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -715,7 +716,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -744,7 +745,8 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
                     }
                     else if(shift) {
-                        if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i))) {
+                        char* str = vector_item(SFD(nextin).fdbuf.mLines, i);
+                        if(!fd_write(nextout, str, strlen(str))) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
@@ -755,7 +757,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
 
-                if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+                if(SFD(nextin).fdbuf.mBufLen == 0) {
                     runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
                 }
                 else {
@@ -802,9 +804,9 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 if(ary && TYPE(ary) == T_VECTOR) {
                     char buf[128];
-                    snprintf(buf, 128, "%d\n", vector_size(ary));
+                    int size = snprintf(buf, 128, "%d\n", vector_size(ary));
 
-                    if(!fd_write(nextout, buf)) {
+                    if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -845,12 +847,12 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                             sObject* var = vector_item(ary, num);
 
-                            if(!fd_write(nextout, string_c_str(var))) {
+                            if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -876,13 +878,13 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         for(j=0; j<vector_size(ary); j++) {
                             sObject* var = vector_item(ary, j);
 
-                            if(!fd_write(nextout, string_c_str(var))) {
+                            if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
 
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -930,7 +932,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             vector_mass_insert(ary, append, appended_ary, max);
             FREE(appended_ary);
             
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -956,7 +958,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 vector_add(new_ary, new_var);
             }
             
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1002,9 +1004,9 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 if(hash && TYPE(hash) == T_HASH) {
                     char buf[128];
-                    snprintf(buf, 128, "%d\n", hash_count(hash));
+                    int size = snprintf(buf, 128, "%d\n", hash_count(hash));
 
-                    if(!fd_write(nextout, buf)) {
+                    if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -1035,12 +1037,12 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         sObject* var = hash_item(hash, argument);
 
                         if(var) {
-                            if(!fd_write(nextout, string_c_str(var))) {
+                            if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -1067,22 +1069,22 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             char* key = hash_loop_key(it);
                             sObject* var = hash_loop_item(it);
 
-                            if(!fd_write(nextout, key)) {
+                            if(!fd_write(nextout, key, strlen(key))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, string_c_str(var))) {
+                            if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
                             }
-                            if(!fd_write(nextout, field)) {
+                            if(!fd_write(nextout, field, strlen(field))) {
                                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                                 return FALSE;
@@ -1129,7 +1131,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
                 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1162,7 +1164,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
                 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1205,12 +1207,12 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 char* env = getenv(command->mArgsRuntime[i]);
                 if(env) {
-                    if(!fd_write(nextout, env)) {
+                    if(!fd_write(nextout, env, strlen(env))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
-                    if(!fd_write(nextout, field)) {
+                    if(!fd_write(nextout, field, strlen(field))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -1241,7 +1243,8 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
-                    if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i-1))) {
+                    char* str = vector_item(SFD(nextin).fdbuf.mLines, i-1);
+                    if(!fd_write(nextout, str, strlen(str))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -1249,7 +1252,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
+            if(SFD(nextin).fdbuf.mBufLen == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
