@@ -437,6 +437,39 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
 
             skip_spaces(p);
         }
+        /// redirect out ///
+        else if(**p == '>') {
+            if(buf->mBuf[0] != 0) {
+                /// message passing ///
+                if(buf->mMessagePassing) {
+                    if(!sCommand_add_message(command, MANAGED buf->mBuf)) {
+                        err_msg("too many message", sname, *sline, buf->mBuf);
+                        FREE(buf->mBuf);
+                        return FALSE;
+                    }
+                }
+                else if(!add_argument_to_command(buf, command, sname, sline)) {
+                    return FALSE;
+                }
+
+                memset(buf, 0, sizeof(sBuf));
+                buf->mBuf = MALLOC(64);
+                buf->mBuf[0] = 0;
+                buf->mSize = 64;
+            }
+
+            (*p)++;
+
+            if(**p == '>') {
+                buf->mRedirect = REDIRECT_APPEND;
+                (*p)++;
+            }
+            else {
+                buf->mRedirect = REDIRECT_OUT;
+            }
+
+            skip_spaces(p);
+        }
         /// redirect in ///
         else if(**p == '<') {
             if(buf->mBuf[0] != 0) {
@@ -472,7 +505,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             buf->mGlob = TRUE;
         }
         /// message passing ///
-        else if(command->mArgsNum == 0 && **p == '-' && *(*p+1) == '>') {
+        else if(command->mArgsNum == 0 && **p == ':' && *(*p+1) == ':') {
             (*p)+=2;
             buf->mMessagePassing = TRUE;
             break;
@@ -562,32 +595,35 @@ void tilda_expasion(char* file, ALLOC char** file2)
     *file2 = STRDUP(result);
 }
 
-static BOOL glob_expasion(sBuf* buf, sCommand* command, char* sname, int* sline)
+static BOOL glob_expasion(sBuf* buf, ALLOC char** head, ALLOC char*** tails, sCommand* command, char* sname, int* sline)
 {
     glob_t result;
-    int rc;
-    if(buf->mTilda) {  // tilda expasion
-        char* buf2;
-        tilda_expasion(buf->mBuf, ALLOC &buf2);
-
-        rc = glob(buf2, 0, NULL, &result);
-        FREE(buf2);
-    }
-    else {
-        rc = glob(buf->mBuf, 0, NULL, &result);
-    }
+    int rc = glob(buf->mBuf, 0, NULL, &result);
 
     if(rc == GLOB_NOSPACE) {
         err_msg("read_one_argument: out of space during glob operation", sname, *sline, buf->mBuf);
         return FALSE;
     }
     else if(rc == GLOB_NOMATCH) {
-        sCommand_add_arg_to_command(command, MANAGED STRDUP(buf->mBuf), 0);
+        *head = STRDUP(buf->mBuf);
+
+        *tails = NULL;
     }
     else {
-        int i;
-        for(i=0; i<result.gl_pathc; i++) {
-            sCommand_add_arg_to_command(command, MANAGED STRDUP(result.gl_pathv[i]), 0);
+        if(result.gl_pathc > 0) {
+            *head = STRDUP(result.gl_pathv[0]);
+
+            *tails = MALLOC(sizeof(char*)*result.gl_pathc);
+
+            int i;
+            for(i=1; i<result.gl_pathc; i++) {
+                (*tails)[i-1] = STRDUP(result.gl_pathv[i]);
+            }
+            (*tails)[i-1] = 0;
+        }
+        else {
+            *head = NULL;
+            *tails = NULL;
         }
     }
 
@@ -651,27 +687,86 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
 
 static BOOL add_argument_to_command(sBuf* buf, sCommand* command, char* sname, int* sline)
 {
-    /// env ///
-    if(buf->mEnv) {
-        sCommand_add_arg_to_command(command, MANAGED buf->mBuf, TRUE);
+    /// tilda expression ///
+    if(buf->mTilda) {
+        char* buf2;
+        tilda_expasion(buf->mBuf, ALLOC &buf2);
+        FREE(buf->mBuf);
+        buf->mBuf = buf2;
+        buf->mTilda = FALSE;
     }
+
     /// redirect ///
-    else if(buf->mRedirect) {
-        sCommand_add_redirect_to_command(command, MANAGED buf->mBuf, buf->mRedirect);
+    if(buf->mRedirect) {
+        if(buf->mGlob && command->mArgsNum > 0) {
+            char* head;
+            char** globs;
+
+            if(!glob_expasion(buf, ALLOC &head, ALLOC &globs, command, sname, sline)) {
+                if(head) FREE(head);
+                if(globs) {
+                    while(*globs) {
+                        FREE(*globs);
+                        globs++;
+                    }
+                    FREE(globs);
+                }
+                FREE(buf->mBuf);
+                return FALSE;
+            }
+
+            if(head) {
+                sCommand_add_redirect_to_command(command, MANAGED head, buf->mRedirect);
+            }
+
+            if(globs) {
+                char** globs2 = globs;
+                while(*globs2) {
+                    sCommand_add_arg_to_command(command, MANAGED *globs2, 0);
+                    globs2++;
+                }
+
+                FREE(globs);
+            }
+
+            FREE(buf->mBuf);
+        }
+        else {
+            sCommand_add_redirect_to_command(command, MANAGED buf->mBuf, buf->mRedirect);
+        }
     }
     /// expand glob ///
     else if(buf->mGlob && command->mArgsNum > 0) {
-        if(!glob_expasion(buf, command, sname, sline)) {
+        char* head;
+        char** globs;
+
+        if(!glob_expasion(buf, ALLOC &head, ALLOC &globs, command, sname, sline)) {
+            if(head) FREE(head);
+            if(globs) {
+                while(*globs) {
+                    FREE(*globs);
+                    globs++;
+                }
+                FREE(globs);
+            }
             FREE(buf->mBuf);
             return FALSE;
         }
-        FREE(buf->mBuf);
-    }
-    /// tilda expression ///
-    else if(buf->mTilda) {
-        char* buf2;
-        tilda_expasion(buf->mBuf, ALLOC &buf2);
-        sCommand_add_arg_to_command(command, MANAGED buf2, 0);
+
+        if(head) {
+            sCommand_add_arg_to_command(command, MANAGED head, 0);
+        }
+
+        if(globs) {
+            char** globs2 = globs;
+            while(*globs2) {
+                sCommand_add_arg_to_command(command, MANAGED *globs2, 0);
+                globs2++;
+            }
+
+            FREE(globs);
+        }
+
         FREE(buf->mBuf);
     }
     /// add buf to command ///
@@ -841,47 +936,6 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
 
         SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_STATMENT_HEAD;
 
-/*
-        /// write ///
-        if(command->mArgsNum > 1 && strcmp(command->mArgs[0], "write") == 0) {
-            if(statment->mCommandsNum > 1) {
-                int append_option_num = -1;
-                char* file_name = NULL;
-                int i;
-                for(i=1; i<command->mArgsNum; i++) {
-                    if(command->mArgs[i][0] == PARSER_MAGIC_NUMBER_OPTION && strcmp(command->mArgs[i] + 1, "append") == 0) {
-                        append_option_num = i;
-                    }
-                    else {
-                        file_name = command->mArgs[i];
-                    }
-                }
-
-                if(file_name == NULL) {
-                    err_msg("invalid write using", sname, *sline, NULL);
-                    return FALSE;
-                }
-                else {
-                    sCommand* prev_command = statment->mCommands + statment->mCommandsNum-2;
-                    if(append_option_num != -1) {
-                        sCommand_add_redirect_to_command(prev_command, MANAGED STRDUP(file_name), REDIRECT_APPEND);
-                    }
-                    else {
-                        sCommand_add_redirect_to_command(prev_command, MANAGED STRDUP(file_name), REDIRECT_OUT);
-                    }
-
-                    sCommand_delete(command);
-                    memset(command, 0, sizeof(sCommand));
-                    statment->mCommandsNum--;
-                }
-            }
-            else {
-                err_msg("invalid write using", sname, *sline, NULL);
-                return FALSE;
-            }
-        }
-        else
-*/
         /// There is no argument, so delete last command in block ///
         if(command->mArgsNum == 0)
         {
