@@ -63,7 +63,7 @@ static char* message_completion(const char* text, int stat)
         }
 
         string_put(messages, command->mMessages[0]);
-        string_push_back(messages, "->");
+        string_push_back(messages, "::");
 
         if(object && TYPE(object) == T_UOBJECT) {
             int i;
@@ -71,7 +71,7 @@ static char* message_completion(const char* text, int stat)
                 object = hash_item(object, command->mMessages[i]);
                 if(object == NULL || TYPE(object) != T_UOBJECT) break;
                 string_push_back(messages, command->mMessages[i]);
-                string_push_back(messages, "->");
+                string_push_back(messages, "::");
             }
 
             if(object && TYPE(object) == T_UOBJECT) {
@@ -88,7 +88,7 @@ static char* message_completion(const char* text, int stat)
 
                         sObject* candidate2 = STRING_NEW_STACK(string_c_str(messages));
                         string_push_back(candidate2, key);
-                        string_push_back(candidate2, "->");
+                        string_push_back(candidate2, "::");
 
                         vector_add(gCompletionArray, string_c_str(candidate2));
                     }
@@ -113,7 +113,7 @@ static char* message_completion(const char* text, int stat)
 
         if(!strncmp(text, candidate, wordlen)) {
             int len = strlen(candidate);
-            if(len > 2 && candidate[len-2] == '-' && candidate[len-1] == '>') {
+            if(len > 2 && candidate[len-2] == ':' && candidate[len-1] == ':') {
                 rl_completion_append_character = 0;
             }
             else {
@@ -151,7 +151,7 @@ static char* all_program_completion(const char* text, int stat)
                         }
                         else {
                             sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
-                            string_push_back(candidate2, "->");
+                            string_push_back(candidate2, "::");
                             vector_add(gCompletionArray, string_c_str(candidate2));
                         }
                     }
@@ -208,7 +208,7 @@ static char* program_completion(const char* text, int stat)
                         }
                         else {
                             sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
-                            string_push_back(candidate2, "->");
+                            string_push_back(candidate2, "::");
                             vector_add(gCompletionArray, string_c_str(candidate2));
                         }
                     }
@@ -236,7 +236,7 @@ static char* program_completion(const char* text, int stat)
 
         if(!strncmp(text, candidate, wordlen)) {
             int len = strlen(candidate);
-            if(len > 2 && candidate[len-2] == '-' && candidate[len-1] == '>') {
+            if(len > 2 && candidate[len-2] == ':' && candidate[len-1] == ':') {
                 rl_completion_append_character = 0;
             }
             else {
@@ -389,7 +389,7 @@ static char* env_completion(const char* text, int stat_)
 
         if(!strncmp(text, candidate, wordlen)) {
             int len = strlen(candidate);
-            if(len > 2 && candidate[len-2] == '-' && candidate[len-1] == '>') {
+            if(len > 2 && candidate[len-2] == ':' && candidate[len-1] == ':') {
                 rl_completion_append_character = 0;
             }
             else {
@@ -565,7 +565,7 @@ char** readline_on_complete(const char* text, int start, int end)
             }
             else if(ucompletion && TYPE(ucompletion) == T_COMPLETION) {
                 sObject* nextin = FD_NEW_STACK(kFDKindBuf, 0);
-                if(!fd_write(nextin, string_c_str(cmdline))) {
+                if(!fd_write(nextin, string_c_str(cmdline), string_length(cmdline))) {
                     stack_end_stack();
                     return NULL;
                 }
@@ -642,7 +642,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* str = STRING_NEW_GC("", FALSE);
             char* p = command->mArgsRuntime[i];
             while(*p) {
-                if(*p == '-' && *(p+1) == '>') {
+                if(*p == ':' && *(p+1) == ':') {
                     if(string_c_str(str)[0] == 0) {
                         err_msg("invalid object name", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
@@ -791,7 +791,7 @@ static int readline_macro(int count, int key)
     int rcode = 0;
     sObject* block = BLOCK_NEW_STACK();
     int sline = 1;
-    if(parse("root->macro", "macro", &sline, block, NULL)) {
+    if(parse("root::macro", "macro", &sline, block, NULL)) {
         xyzsh_set_signal();
 
         sObject* fun = FUN_NEW_STACK(NULL);
@@ -802,7 +802,7 @@ static int readline_macro(int count, int key)
 
         sObject* nextin2 = FD_NEW_STACK(kFDKindBuf, 0);
 
-        (void)fd_write3(nextin2, rl_line_buffer, rl_point);
+        (void)fd_write(nextin2, rl_line_buffer, rl_point);
 
         if(!run(block, nextin2, nextout2, &rcode, gRootObject, fun)) {
             if(rcode == RCODE_BREAK) {
@@ -1131,9 +1131,9 @@ void xyzsh_readline_init(BOOL runtime_script)
     rl_attempted_completion_function = readline_on_complete;
 //    rl_completion_entry_function = readline_filename_completion_null_generator;
     rl_completer_quote_characters = "\"'";
-    rl_completer_word_break_characters = " \t\n\"'|!&;()$%<";
+    rl_completer_word_break_characters = " \t\n\"'|!&;()$%<>";
     rl_completion_append_character= ' ';
-    rl_filename_quote_characters = " \t\n\"'|!&;()$%<";
+    rl_filename_quote_characters = " \t\n\"'|!&;()$%<>:";
     rl_filename_quoting_function = bash_quote_filename;
     rl_filename_dequoting_function = (rl_dequote_func_t*)bash_dequote_filename;
 
